@@ -3,10 +3,13 @@
 
 import types
 import os
+import sys
+import inspect
+import six
 
 from .cmdparse import CmdOption, TaskParse
 from .exceptions import CatchedException, InvalidTask
-from .action import create_action
+from .action import create_action, PythonAction
 
 
 class Task(object):
@@ -43,10 +46,10 @@ class Task(object):
     """
 
     DEFAULT_VERBOSITY = 1
-
+    string_types = (str, ) if six.PY3 else (str, unicode)
     # list of valid types/values for each task attribute.
-    valid_attr = {'basename': ((str, unicode), ()),
-                  'name': ((str, unicode), ()),
+    valid_attr = {'basename': (string_types, ()),
+                  'name': (string_types, ()),
                   'actions': ((list, tuple), (None,)),
                   'file_dep': ((list, tuple), ()),
                   'task_dep': ((list, tuple), ()),
@@ -56,7 +59,7 @@ class Task(object):
                   'setup': ((list, tuple), ()),
                   'clean': ((list, tuple), (True,)),
                   'teardown': ((list, tuple), ()),
-                  'doc': ((str, unicode), (None,)),
+                  'doc': (string_types, (None,)),
                   'params': ((list, tuple,), ()),
                   'verbosity': ((), (None,0,1,2,)),
                   'getargs': ((dict,), ()),
@@ -181,7 +184,7 @@ class Task(object):
     def _expand_file_dep(self, file_dep):
         """put input into file_dep"""
         for dep in file_dep:
-            if not isinstance(dep, basestring):
+            if not isinstance(dep, six.string_types):
                 raise InvalidTask("%s. file_dep must be a str got '%r' (%s)" %
                                   (self.name, dep, type(dep)))
             self.file_dep.add(dep)
@@ -216,7 +219,7 @@ class Task(object):
                    }
     def update_deps(self, deps):
         """expand all kinds of dep input"""
-        for dep, dep_values in deps.iteritems():
+        for dep, dep_values in six.iteritems(deps):
             if dep not in self._expand_map:
                 continue
             self._expand_map[dep](self, dep_values)
@@ -226,6 +229,8 @@ class Task(object):
         """put default values on options. this will be overwritten, if params
         options were passed on the command line.
         """
+        # FIXME call _init_options just once and not check if this.options is None
+        # because its value may be set in a different place... (like uptodate())
         if self.options is None:
             # ignore positional parameters
             self.options = self.taskcmd.parse('')[0]
@@ -234,11 +239,11 @@ class Task(object):
     def _init_getargs(self):
         """task getargs attribute define implicit task dependencies"""
         self._init_options()
-        for arg_name, desc in self.getargs.iteritems():
+        for arg_name, desc in six.iteritems(self.getargs):
 
             # tuple (task_id, key_name)
             parts = desc
-            if isinstance(parts, basestring) or len(parts) != 2:
+            if isinstance(parts, six.string_types) or len(parts) != 2:
                 msg = ("Taskid '%s' - Invalid format for getargs of '%s'.\n" %
                        (self.name, arg_name) +
                        "Should be tuple with 2 elements " +
@@ -346,35 +351,23 @@ class Task(object):
         self._init_options()
         # if clean is True remove all targets
         if self._remove_targets is True:
-            files = [path for path in self.targets if os.path.isfile(path)]
-            dirs = [path for path in self.targets if os.path.isdir(path)]
-
-            # remove all files
-            for file_ in files:
-                msg = "%s - removing file '%s'\n" % (self.name, file_)
-                outstream.write(msg)
-                if not dryrun:
-                    os.remove(file_)
-
-            # remove all directories (if empty)
-            for dir_ in dirs:
-                if os.listdir(dir_):
-                    msg = "%s - cannot remove (it is not empty) '%s'\n"
-                    outstream.write(msg % (self.name, dir_))
-                else:
-                    msg = "%s - removing dir '%s'\n"
-                    outstream.write(msg % (self.name, dir_))
-                    if not dryrun:
-                        os.rmdir(dir_)
-
+            clean_targets(self, dryrun)
         else:
             # clean contains a list of actions...
             for action in self.clean_actions:
                 msg = "%s - executing '%s'\n"
                 outstream.write(msg % (self.name, action))
-                if not dryrun:
-                    action.execute()
 
+                # add extra arguments used by clean actions
+                if isinstance(action, PythonAction):
+                    action_args = inspect.getargspec(action.py_callable).args
+                    if 'dryrun' in action_args:
+                        action.kwargs['dryrun'] = dryrun
+
+                if not dryrun:
+                    result = action.execute(out=outstream)
+                    if isinstance(result, CatchedException):
+                        sys.stderr.write(str(result))
 
     def title(self):
         """String representation on output.
@@ -417,6 +410,7 @@ class Task(object):
         return self.name < other.name
 
 
+
 def dict_to_task(task_dict):
     """Create a task instance from dictionary.
 
@@ -432,11 +426,35 @@ def dict_to_task(task_dict):
                           (task_dict['name'],task_dict))
 
     # user friendly. dont go ahead with invalid input.
-    task_attrs = task_dict.keys()
-    valid_attrs = set(Task.valid_attr.iterkeys())
+    task_attrs = list(six.iterkeys(task_dict))
+    valid_attrs = set(six.iterkeys(Task.valid_attr))
     for key in task_attrs:
         if key not in valid_attrs:
             raise InvalidTask("Task %s contains invalid field: '%s'"%
                               (task_dict['name'],key))
 
     return Task(**task_dict)
+
+
+
+def clean_targets(task, dryrun):
+    """remove all targets from a task"""
+    files = [path for path in task.targets if os.path.isfile(path)]
+    dirs = [path for path in task.targets if os.path.isdir(path)]
+
+    # remove all files
+    for file_ in files:
+        six.print_("%s - removing file '%s'" % (task.name, file_))
+        if not dryrun:
+            os.remove(file_)
+
+    # remove all directories (if empty)
+    for dir_ in dirs:
+        if os.listdir(dir_):
+            msg = "%s - cannot remove (it is not empty) '%s'"
+            six.print_(msg % (task.name, dir_))
+        else:
+            msg = "%s - removing dir '%s'"
+            six.print_(msg % (task.name, dir_))
+            if not dryrun:
+                os.rmdir(dir_)
