@@ -2,18 +2,19 @@
 
 import os
 import time
-import anydbm
+import six
 
 import pytest
 
 from doit.task import Task
 from doit.dependency import get_md5, md5sum, check_modified, UptodateCalculator
-from doit.dependency import JsonDependency, DbmDependency, DbmDB
+from doit.dependency import DbmDB, DatabaseException
+from doit.dependency import JsonDependency, DbmDependency, SqliteDependency
 from .conftest import get_abspath, depfile
 
 
 def test_unicode_md5():
-    data = u"我"
+    data = six.u("我")
     # no exception is raised
     assert get_md5(data)
 
@@ -42,136 +43,136 @@ def test_md5():
 
 
 
-# test parametrization, execute tests for all DB backends
-pytest.fixture(params=[JsonDependency, DbmDependency])(depfile)
-
+# test parametrization, execute tests for all DB backends.
+# create a separate fixture to be used only by this module
+# because only here it is required to test with all backends
+@pytest.fixture
+def pdepfile(request):
+    return depfile(request)
+pytest.fixture(params=[JsonDependency, DbmDependency, SqliteDependency])(pdepfile)
 
 # FIXME there was major refactor breaking classes from dependency,
 # unit-tests could be more specific to base classes.
 
 class TestDependencyDb(object):
     # adding a new value to the DB
-    def test_get_set(self, depfile):
-        depfile._set("taskId_X","dependency_A","da_md5")
-        value = depfile._get("taskId_X","dependency_A")
+    def test_get_set(self, pdepfile):
+        pdepfile._set("taskId_X","dependency_A","da_md5")
+        value = pdepfile._get("taskId_X","dependency_A")
         assert "da_md5" == value, value
 
-    def test_get_set_unicode_name(self, depfile):
-        depfile._set(u"taskId_我","dependency_A","da_md5")
-        value = depfile._get(u"taskId_我","dependency_A")
+    def test_get_set_unicode_name(self, pdepfile):
+        pdepfile._set(six.u("taskId_我"),"dependency_A","da_md5")
+        value = pdepfile._get(six.u("taskId_我"),"dependency_A")
         assert "da_md5" == value, value
 
     #
-    def test_dump(self, depfile):
+    def test_dump(self, pdepfile):
         # save and close db
-        depfile._set("taskId_X","dependency_A","da_md5")
-        depfile.close()
+        pdepfile._set("taskId_X","dependency_A","da_md5")
+        pdepfile.close()
 
         # open it again and check the value
-        d2 = depfile.__class__(depfile.name)
+        d2 = pdepfile.__class__(pdepfile.name)
 
         value = d2._get("taskId_X","dependency_A")
         assert "da_md5" == value, value
 
-    def test_corrupted_file(self, depfile):
-        if depfile.__class__==DbmDependency and depfile.whichdb is None: # pragma: no cover
+    def test_corrupted_file(self, pdepfile):
+        if pdepfile.__class__==DbmDependency and pdepfile.whichdb is None: # pragma: no cover
             pytest.skip('dumbdbm too dumb to detect db corruption')
 
         # create some corrupted files
-        for name_ext in depfile.name_ext:
-            full_name = depfile.name + name_ext
+        for name_ext in pdepfile.name_ext:
+            full_name = pdepfile.name + name_ext
             fd = open(full_name, 'w')
             fd.write("""{"x": y}""")
             fd.close()
-        if isinstance(anydbm.error, Exception): # pragma: no cover
-            exceptions = (ValueError, anydbm.error)
-        else:
-            exceptions = (ValueError,) + anydbm.error
-        pytest.raises(exceptions, depfile.__class__, depfile.name)
+        pytest.raises(DatabaseException, pdepfile.__class__, pdepfile.name)
 
-    def test_corrupted_file_unrecognized_excep(self, monkeypatch, depfile):
-        if isinstance(depfile, JsonDependency):
+    def test_corrupted_file_unrecognized_excep(self, monkeypatch, pdepfile):
+        if isinstance(pdepfile, JsonDependency):
             pytest.skip('test doesnt apply to JsonDependency')
-        if depfile.whichdb is None: # pragma: no cover
+        if pdepfile.whichdb is None: # pragma: no cover
             pytest.skip('dumbdbm too dumb to detect db corruption')
 
         # create some corrupted files
-        for name_ext in depfile.name_ext:
-            full_name = depfile.name + name_ext
+        for name_ext in pdepfile.name_ext:
+            full_name = pdepfile.name + name_ext
             fd = open(full_name, 'w')
             fd.write("""{"x": y}""")
             fd.close()
         monkeypatch.setattr(DbmDB, 'DBM_CONTENT_ERROR_MSG', 'xxx')
-        pytest.raises(anydbm.error, depfile.__class__, depfile.name)
+        pytest.raises(DatabaseException, pdepfile.__class__, pdepfile.name)
 
     # _get must return None if entry doesnt exist.
-    def test_getNonExistent(self, depfile):
-        assert depfile._get("taskId_X","dependency_A") == None
+    def test_getNonExistent(self, pdepfile):
+        assert pdepfile._get("taskId_X","dependency_A") == None
 
 
-    def test_in(self, depfile):
-        depfile._set("taskId_ZZZ","dep_1","12")
-        assert depfile._in("taskId_ZZZ")
-        assert not depfile._in("taskId_hohoho")
+    def test_in(self, pdepfile):
+        pdepfile._set("taskId_ZZZ","dep_1","12")
+        assert pdepfile._in("taskId_ZZZ")
+        assert not pdepfile._in("taskId_hohoho")
 
 
-    def test_remove(self, depfile):
-        depfile._set("taskId_ZZZ","dep_1","12")
-        depfile._set("taskId_ZZZ","dep_2","13")
-        depfile._set("taskId_YYY","dep_1","14")
-        depfile.remove("taskId_ZZZ")
-        assert None == depfile._get("taskId_ZZZ","dep_1")
-        assert None == depfile._get("taskId_ZZZ","dep_2")
-        assert "14" == depfile._get("taskId_YYY","dep_1")
+    def test_remove(self, pdepfile):
+        pdepfile._set("taskId_ZZZ","dep_1","12")
+        pdepfile._set("taskId_ZZZ","dep_2","13")
+        pdepfile._set("taskId_YYY","dep_1","14")
+        pdepfile.remove("taskId_ZZZ")
+        assert None == pdepfile._get("taskId_ZZZ","dep_1")
+        assert None == pdepfile._get("taskId_ZZZ","dep_2")
+        assert "14" == pdepfile._get("taskId_YYY","dep_1")
 
 
     # special test for DBM backend and "dirty"/caching mechanism
-    def test_remove_from_non_empty_file(self, depfile):
+    def test_remove_from_non_empty_file(self, pdepfile):
         # 1 - put 2 tasks of file
-        depfile._set("taskId_XXX", "dep_1", "x")
-        depfile._set("taskId_YYY", "dep_1", "x")
-        depfile.close()
+        pdepfile._set("taskId_XXX", "dep_1", "x")
+        pdepfile._set("taskId_YYY", "dep_1", "x")
+        pdepfile.close()
         # 2 - re-open and remove one task
-        reopened = depfile.__class__(depfile.name)
+        reopened = pdepfile.__class__(pdepfile.name)
         reopened.remove("taskId_YYY")
         reopened.close()
         # 3 - re-open again and check task was really removed
-        reopened2 = depfile.__class__(depfile.name)
+        reopened2 = pdepfile.__class__(pdepfile.name)
         assert reopened2._in("taskId_XXX")
         assert not reopened2._in("taskId_YYY")
 
 
-    def test_remove_all(self, depfile):
-        depfile._set("taskId_ZZZ","dep_1","12")
-        depfile._set("taskId_ZZZ","dep_2","13")
-        depfile._set("taskId_YYY","dep_1","14")
-        depfile.remove_all()
-        assert None == depfile._get("taskId_ZZZ","dep_1")
-        assert None == depfile._get("taskId_ZZZ","dep_2")
-        assert None == depfile._get("taskId_YYY","dep_1")
+    def test_remove_all(self, pdepfile):
+        pdepfile._set("taskId_ZZZ","dep_1","12")
+        pdepfile._set("taskId_ZZZ","dep_2","13")
+        pdepfile._set("taskId_YYY","dep_1","14")
+        pdepfile.remove_all()
+        assert None == pdepfile._get("taskId_ZZZ","dep_1")
+        assert None == pdepfile._get("taskId_ZZZ","dep_2")
+        assert None == pdepfile._get("taskId_YYY","dep_1")
 
 
 
 class TestSaveSuccess(object):
 
-    def test_save_result(self, depfile):
+    def test_save_result(self, pdepfile):
         t1 = Task('t_name', None)
         t1.result = "result"
-        depfile.save_success(t1)
-        assert get_md5("result") == depfile._get(t1.name, "result:")
+        pdepfile.save_success(t1)
+        assert get_md5("result") == pdepfile._get(t1.name, "result:")
 
-    def test_save_resultNone(self, depfile):
+    def test_save_resultNone(self, pdepfile):
         t1 = Task('t_name', None)
-        depfile.save_success(t1)
-        assert None is depfile._get(t1.name, "result:")
+        pdepfile.save_success(t1)
+        assert None is pdepfile._get(t1.name, "result:")
 
-    def test_save_result_dict(self, depfile):
+    def test_save_result_dict(self, pdepfile):
         t1 = Task('t_name', None)
         t1.result = {'d': "result"}
-        depfile.save_success(t1)
-        assert {'d': "result"} == depfile._get(t1.name, "result:")
+        pdepfile.save_success(t1)
+        assert {'d': "result"} == pdepfile._get(t1.name, "result:")
 
-    def test_save_file_md5(self, depfile):
+    def test_save_file_md5(self, pdepfile):
         # create a test dependency file
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
@@ -180,25 +181,25 @@ class TestSaveSuccess(object):
 
         # save it
         t1 = Task("taskId_X", None, [filePath])
-        depfile.save_success(t1)
+        pdepfile.save_success(t1)
         expected = "a1bb792202ce163b4f0d17cb264c04e1"
-        value = depfile._get("taskId_X",filePath)
+        value = pdepfile._get("taskId_X",filePath)
         assert os.path.getmtime(filePath) == value[0] # timestamp
         assert 39 == value[1] # size
         assert expected == value[2] # MD5
 
-    def test_save_skip(self, depfile, monkeypatch):
-        #self.test_save_file_md5(depfile)
+    def test_save_skip(self, pdepfile, monkeypatch):
+        #self.test_save_file_md5(pdepfile)
         filePath = get_abspath("data/dependency1")
         t1 = Task("taskId_X", None, [filePath])
-        depfile._set(t1.name, filePath, (345, 0, "fake"))
+        pdepfile._set(t1.name, filePath, (345, 0, "fake"))
         monkeypatch.setattr(os.path, 'getmtime', lambda x: 345)
         # save but md5 is not modified
-        depfile.save_success(t1)
-        got = depfile._get("taskId_X", filePath)
+        pdepfile.save_success(t1)
+        got = pdepfile._get("taskId_X", filePath)
         assert "fake" == got[2]
 
-    def test_save_files(self, depfile):
+    def test_save_files(self, pdepfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -207,68 +208,69 @@ class TestSaveSuccess(object):
         ff = open(filePath2,"w")
         ff.write("part2")
         ff.close()
+        assert pdepfile._get("taskId_X",filePath) is None
+        assert pdepfile._get("taskId_X",filePath2) is None
 
-        assert 0 == len(depfile.backend._db)
         t1 = Task("taskId_X", None, [filePath,filePath2])
-        depfile.save_success(t1)
-        assert depfile._get("taskId_X",filePath) is not None
-        assert depfile._get("taskId_X",filePath2) is not None
+        pdepfile.save_success(t1)
+        assert pdepfile._get("taskId_X",filePath) is not None
+        assert pdepfile._get("taskId_X",filePath2) is not None
 
-    def test_save_values(self, depfile):
+    def test_save_values(self, pdepfile):
         t1 = Task('t1', None)
         t1.values = {'x':5, 'y':10}
-        depfile.save_success(t1)
-        assert {'x':5, 'y':10} == depfile._get("t1", "_values_:")
+        pdepfile.save_success(t1)
+        assert {'x':5, 'y':10} == pdepfile._get("t1", "_values_:")
 
 
 class TestGetValue(object):
-    def test_all_values(self, depfile):
+    def test_all_values(self, pdepfile):
         t1 = Task('t1', None)
         t1.values = {'x':5, 'y':10}
-        depfile.save_success(t1)
-        assert {'x':5, 'y':10} == depfile.get_values('t1')
+        pdepfile.save_success(t1)
+        assert {'x':5, 'y':10} == pdepfile.get_values('t1')
 
-    def test_ok(self, depfile):
+    def test_ok(self, pdepfile):
         t1 = Task('t1', None)
         t1.values = {'x':5, 'y':10}
-        depfile.save_success(t1)
-        assert 5 == depfile.get_value('t1', 'x')
+        pdepfile.save_success(t1)
+        assert 5 == pdepfile.get_value('t1', 'x')
 
-    def test_ok_dot_on_task_name(self, depfile):
+    def test_ok_dot_on_task_name(self, pdepfile):
         t1 = Task('t1:a.ext', None)
         t1.values = {'x':5, 'y':10}
-        depfile.save_success(t1)
-        assert 5 == depfile.get_value('t1:a.ext', 'x')
+        pdepfile.save_success(t1)
+        assert 5 == pdepfile.get_value('t1:a.ext', 'x')
 
-    def test_invalid_taskid(self, depfile):
+    def test_invalid_taskid(self, pdepfile):
         t1 = Task('t1', None)
         t1.values = {'x':5, 'y':10}
-        depfile.save_success(t1)
-        pytest.raises(Exception, depfile.get_value, 'nonono', 'x')
+        pdepfile.save_success(t1)
+        pytest.raises(Exception, pdepfile.get_value, 'nonono', 'x')
 
-    def test_invalid_key(self, depfile):
+    def test_invalid_key(self, pdepfile):
         t1 = Task('t1', None)
         t1.values = {'x':5, 'y':10}
-        depfile.save_success(t1)
-        pytest.raises(Exception, depfile.get_value, 't1', 'z')
+        pdepfile.save_success(t1)
+        pytest.raises(Exception, pdepfile.get_value, 't1', 'z')
 
 
 
 class TestRemoveSuccess(object):
-    def test_save_result(self, depfile):
+    def test_save_result(self, pdepfile):
         t1 = Task('t_name', None)
         t1.result = "result"
-        depfile.save_success(t1)
-        assert get_md5("result") == depfile._get(t1.name, "result:")
-        depfile.remove_success(t1)
-        assert None is depfile._get(t1.name, "result:")
+        pdepfile.save_success(t1)
+        assert get_md5("result") == pdepfile._get(t1.name, "result:")
+        pdepfile.remove_success(t1)
+        assert None is pdepfile._get(t1.name, "result:")
 
 
 class TestIgnore(object):
-    def test_save_result(self, depfile):
+    def test_save_result(self, pdepfile):
         t1 = Task('t_name', None)
-        depfile.ignore(t1)
-        assert '1' == depfile._get(t1.name, "ignore:")
+        pdepfile.ignore(t1)
+        assert '1' == pdepfile._get(t1.name, "ignore:")
 
 
 class TestCheckModified(object):
@@ -296,16 +298,16 @@ class TestCheckModified(object):
 
 class TestGetStatus(object):
 
-    def test_ignore(self, depfile):
+    def test_ignore(self, pdepfile):
         t1 = Task("t1", None)
         # before ignore
-        assert not depfile.status_is_ignore(t1)
+        assert not pdepfile.status_is_ignore(t1)
         # after ignote
-        depfile.ignore(t1)
-        assert depfile.status_is_ignore(t1)
+        pdepfile.ignore(t1)
+        assert pdepfile.status_is_ignore(t1)
 
 
-    def test_fileDependencies(self, depfile):
+    def test_fileDependencies(self, pdepfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -315,12 +317,12 @@ class TestGetStatus(object):
         t1 = Task("t1", None, dependencies)
 
         # first time execute
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert dependencies == t1.dep_changed
 
         # second time no
-        depfile.save_success(t1)
-        assert 'up-to-date' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
         # FIXME - mock timestamp
@@ -331,29 +333,29 @@ class TestGetStatus(object):
         ff.close()
 
         # execute again
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert dependencies == t1.dep_changed
 
 
-    def test_file_dependency_not_exist(self, depfile):
+    def test_file_dependency_not_exist(self, pdepfile):
         filePath = get_abspath("data/dependency_not_exist")
         t1 = Task("t1", None, [filePath])
-        pytest.raises(Exception, depfile.get_status, t1, {})
+        pytest.raises(Exception, pdepfile.get_status, t1, {})
 
 
     # if there is no dependency the task is always executed
-    def test_noDependency(self, depfile):
+    def test_noDependency(self, pdepfile):
         t1 = Task("t1", None)
         # first time execute
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
         # second too
-        depfile.save_success(t1)
-        assert 'run' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
 
-    def test_UptodateFalse(self, depfile):
+    def test_UptodateFalse(self, pdepfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -362,20 +364,20 @@ class TestGetStatus(object):
         t1 = Task("t1", None, file_dep=[filePath], uptodate=[False])
 
         # first time execute
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
         # second time execute too
-        depfile.save_success(t1)
-        assert 'run' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
-    def test_UptodateTrue(self, depfile):
+    def test_UptodateTrue(self, pdepfile):
         t1 = Task("t1", None, uptodate=[True])
-        depfile.save_success(t1)
-        assert 'up-to-date' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
 
-    def test_UptodateNone(self, depfile):
+    def test_UptodateNone(self, pdepfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -384,21 +386,21 @@ class TestGetStatus(object):
         t1 = Task("t1", None, file_dep=[filePath], uptodate=[None])
 
         # first time execute
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [filePath] == t1.dep_changed
 
         # second time execute too
-        depfile.save_success(t1)
-        assert 'up-to-date' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
 
 
-    def test_UptodateCallable_True(self, depfile):
+    def test_UptodateCallable_True(self, pdepfile):
         def check(task, values): return True
         t1 = Task("t1", None, uptodate=[check])
-        depfile.save_success(t1)
-        assert 'up-to-date' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
 
-    def test_UptodateCallable_False(self, depfile):
+    def test_UptodateCallable_False(self, pdepfile):
         filePath = get_abspath("data/dependency1")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -408,16 +410,16 @@ class TestGetStatus(object):
         t1 = Task("t1", None, file_dep=[filePath], uptodate=[check])
 
         # first time execute
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
         # second time execute too
-        depfile.save_success(t1)
-        assert 'run' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
 
-    def test_UptodateCallable_added_attributes(self, depfile):
+    def test_UptodateCallable_added_attributes(self, pdepfile):
         task_dict = "fake dict"
         class My_uptodate(UptodateCalculator):
             def __call__(self, task, values):
@@ -428,22 +430,22 @@ class TestGetStatus(object):
 
         check = My_uptodate()
         t1 = Task("t1", None, uptodate=[check])
-        assert 'up-to-date' == depfile.get_status(t1, task_dict)
+        assert 'up-to-date' == pdepfile.get_status(t1, task_dict)
 
 
     # if target file does not exist, task is outdated.
-    def test_targets_notThere(self, depfile, dependency1):
+    def test_targets_notThere(self, pdepfile, dependency1):
         target = get_abspath("data/target")
         if os.path.exists(target):
             os.remove(target)
 
         t1 = Task("task x", None, [dependency1], [target])
-        depfile.save_success(t1)
-        assert 'run' == depfile.get_status(t1, {})
+        pdepfile.save_success(t1)
+        assert 'run' == pdepfile.get_status(t1, {})
         assert [dependency1] == t1.dep_changed
 
 
-    def test_targets(self, depfile, dependency1):
+    def test_targets(self, pdepfile, dependency1):
         filePath = get_abspath("data/target")
         ff = open(filePath,"w")
         ff.write("part1")
@@ -453,26 +455,24 @@ class TestGetStatus(object):
         targets = [filePath]
         t1 = Task("task X", None, deps, targets)
 
-        depfile.save_success(t1)
+        pdepfile.save_success(t1)
         # up-to-date because target exist
-        assert 'up-to-date' == depfile.get_status(t1, {})
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
 
 
-    def test_targetFolder(self, depfile, dependency1):
+    def test_targetFolder(self, pdepfile, dependency1):
         # folder not there. task is not up-to-date
         deps = [dependency1]
         folderPath = get_abspath("data/target-folder")
         if os.path.exists(folderPath):
             os.rmdir(folderPath)
         t1 = Task("task x", None, deps, [folderPath])
-        depfile.save_success(t1)
+        pdepfile.save_success(t1)
 
-        assert 'run' == depfile.get_status(t1, {})
+        assert 'run' == pdepfile.get_status(t1, {})
         assert deps == t1.dep_changed
         # create folder. task is up-to-date
         os.mkdir(folderPath)
-        assert 'up-to-date' == depfile.get_status(t1, {})
+        assert 'up-to-date' == pdepfile.get_status(t1, {})
         assert [] == t1.dep_changed
-
-
